@@ -56,23 +56,6 @@ class RecognitionClient(RestClient):
         """
         return super().delete(api_endpoint, data=data, params=self.add_workspace(params))
 
-    def add_workspace(self, data, url=None):
-        """
-        Add workspace uuid to the data.
-        :param data: dictionary/json data which will be send to endpoint
-        :return: modified json data with workspace
-        """
-        if self.workspace != DEFAULT_WORKSPACE:
-            if data is None:
-                data = {}
-
-            # if workspace is already in url then do not create the param
-            if url is not None and self.workspace not in url:
-                data[WORKSPACE] = self.workspace
-            elif url is None:
-                data[WORKSPACE] = self.workspace
-        return data
-
     def get_workspaces(self):
         """
         Get all workspaces accessed by user.
@@ -168,6 +151,12 @@ class RecognitionClient(RestClient):
 
         return None, {STATUS: "Task with this name not found!"}
 
+    def add_label_to_image(self, image_id, label_id):
+        """
+        Add label to the image.
+        """
+        return self.post(IMAGE_ENDPOINT + image_id + "/add-label/", data={LABEL_ID: label_id})
+
     def get_training_images(self, page_url=None, verification=None):
         """
         Get paginated result of images from workspace.
@@ -186,7 +175,7 @@ class RecognitionClient(RestClient):
         return (
             [Image(self.token, self.endpoint, image_json) for image_json in result[RESULTS]],
             result[NEXT],
-            RESULT_OK,
+            {"count": result["count"], STATUS: "ok"},
         )
 
     def training_images_iter(self, verification=None, batch_size=1):
@@ -206,19 +195,17 @@ class RecognitionClient(RestClient):
                 break
             images, next_page, status = self.get_training_images(next_page)
 
-    def modify_images(self, images=None, labels=None, add_labels=None, remove_labels=None):
+    def modify_images(self, images=None, labels=None, add_labels=None, remove_labels=None, test=None, real=None):
         """
         Modifies images in batch. Enter either list of images OR labels to be applied as a filter.
-        :param labels:
-        Filter images by these labels given as a list of IDs
-        :param images:
-        List of image IDs
-        :param add_labels:
-        List of label IDs to assign
-        :param remove_labels:
-        List of label IDs to remove
-        :return: int
-        Number of processed records
+        :param images: [description], defaults to None
+        :param labels: [description], defaults to None
+        :param add_labels: [description], defaults to None
+        :param remove_labels: [description], defaults to None
+        :param test: [description], defaults to None
+        :param real: [description], defaults to None
+        :raises XimilarClientInvalidDataException: [description]
+        :return: [description]
         """
         if images is None:
             images = []
@@ -232,12 +219,26 @@ class RecognitionClient(RestClient):
         if not (labels or images):
             raise XimilarClientInvalidDataException("Either images or labels must be specified")
 
-        return self.post(api_endpoint=IMAGE_ENDPOINT + "update", data={
+        data = {
             "images": images,
             "labels": labels,
             "labels-add": add_labels,
             "labels-remove": remove_labels,
-        })
+        }
+
+        if test is not None:
+            if test:
+                data["mark-test"] = True
+            else:
+                data["unmark-test"] = True
+
+        if real is not None:
+            if real:
+                data["mark-real"] = True
+            else:
+                data["unmark-real"] = True
+
+        return self.post(api_endpoint=IMAGE_ENDPOINT + "update", data=data)
 
     def get_labels_by_substring(self, name):
         """
@@ -331,6 +332,9 @@ class RecognitionClient(RestClient):
 
             image = Image(self.token, self.endpoint, image_json)
 
+            if REAL_IMAGE in record:
+                image.set_real(record[REAL_IMAGE])
+
             if LABELS in record:
                 for label_id in record[LABELS]:
                     image.add_label(label_id)
@@ -418,7 +422,7 @@ class Task(RecognitionClient):
     """
 
     def __init__(self, token, endpoint, task_json):
-        super(Task, self).__init__(token, endpoint)
+        super(Task, self).__init__(token, endpoint, resource_name=None)
 
         self.id = task_json[ID]
         self.name = task_json[NAME]
@@ -512,7 +516,7 @@ class Task(RecognitionClient):
         :param label_id: identification of label
         :return: json/dict result
         """
-        return self.post(TASK_ENDPOINT + self.id + "/add-label/", data={LABEL_ID: label_id})
+        return self.add_label_to_image(self.id, label_id)
 
     def detach_label(self, label_id):
         """
@@ -540,7 +544,7 @@ class Model(RecognitionClient):
     """
 
     def __init__(self, token, endpoint, model_json):
-        super(Model, self).__init__(token, endpoint)
+        super(Model, self).__init__(token, endpoint, resource_name=None)
 
         self.id = model_json[ID]
         self.task_id = model_json[TASK]
@@ -562,7 +566,7 @@ class Label(RecognitionClient):
     """
 
     def __init__(self, token, endpoint, label_json):
-        super(Label, self).__init__(token, endpoint)
+        super(Label, self).__init__(token, endpoint, resource_name=None)
 
         self.id = label_json[ID]
         self.name = label_json[NAME]
@@ -677,15 +681,15 @@ class Image(RecognitionClient):
     """
 
     def __init__(self, token, endpoint, image_json):
-        super(Image, self).__init__(token, endpoint)
+        super(Image, self).__init__(token, endpoint, resource_name=None)
 
         self.id = image_json[ID]
         self.img_path = image_json[IMG_PATH]
         self.thumb_img_path = image_json[THUMB_IMG_PATH]
         self.verifyCount = image_json[VERIFY_COUNT] if VERIFY_COUNT in image_json else -1
         self.workspace = image_json[WORKSPACE] if WORKSPACE in image_json else DEFAULT_WORKSPACE
-        self.img_width = image_json[IMG_WIDTH]
-        self.img_height = image_json[IMG_HEIGHT]
+        self.img_width = image_json[IMG_WIDTH] if IMG_WIDTH in image_json else None
+        self.img_height = image_json[IMG_HEIGHT] if IMG_HEIGHT in image_json else None
         # if the meta data was downloaded from the server, this field is never None
         self.meta_data = (
             None if META_DATA not in image_json else (image_json[META_DATA] if image_json[META_DATA] else {})
@@ -693,6 +697,8 @@ class Image(RecognitionClient):
         # file path after calling download_image on this object
         self._file = None
         self._objects = []
+        self.test_image = image_json[TEST_IMAGE] if TEST_IMAGE in image_json else None
+        self.real_image = image_json[REAL_IMAGE] if REAL_IMAGE in image_json else None
 
     def __str__(self):
         return self.thumb_img_path
@@ -703,6 +709,14 @@ class Image(RecognitionClient):
         """
         return self.remove_image(self.id)
 
+    def set_test(self, test):
+        self.modify_images([self.id], test=test)
+        self.test_image = test
+
+    def set_real(self, real):
+        self.modify_images([self.id], real=real)
+        self.real_image = real
+
     def get_labels(self):
         """
         Get labels assigned to this image.
