@@ -12,10 +12,12 @@ from ximilar.client.constants import (
     COLLECTION_ID,
     PHOTO_SIMILARITY,
     PRODUCT_SIMILARITY,
+    PRODUCT_SIMILARITY_TAGS,
 )
 
 SIMILARITY_PHOTOS = "similarity/photos/v2/"
 SIMILARITY_PRODUCTS = "similarity/products/v2/"
+SIMILARITY_PRODUCTS_TAGS = "similarity/products_tags/v2/"
 SMART_SEARCH = "smart/v2/product/"
 
 SEARCH_OBJ_ENDPOINT = "search_by_object"
@@ -28,25 +30,17 @@ UPDATE = "update"
 RANDOM = "random"
 KNN_VISUAL = "visualKNN"
 PING = "ping"
+RANK_RECORDS = "visualRankRecords"
 
 
 class SimilarityPhotosClient(RestClient):
     def __init__(
-        self,
-        token,
-        collection=None,
-        collection_id=None,
-        endpoint=ENDPOINT + SIMILARITY_PHOTOS,
-        resource_name=PHOTO_SIMILARITY,
+        self, token, collection_id=None, endpoint=ENDPOINT + SIMILARITY_PHOTOS, resource_name=PHOTO_SIMILARITY,
     ):
         super(SimilarityPhotosClient, self).__init__(
             token=token, endpoint=endpoint, max_image_size=512, resource_name=resource_name
         )
-        if collection_id:
-            self.headers[COLLECTION_ID] = collection_id
-        else:
-            self.headers[COLLECTION] = collection
-
+        self.headers[COLLECTION_ID] = collection_id
         self.PREDICT_ENDPOINT = KNN_VISUAL
 
     def construct_data(self, query_record=None, filter=None, k=5, fields_to_return=[_ID]):
@@ -62,10 +56,20 @@ class SimilarityPhotosClient(RestClient):
             data[FILTER] = filter
         return data
 
-    def allRecords(self, size=1000, page=1):
-        result = self.post("allRecords?size=%s&page=%s" % (size, page), data={})
+    def allRecords(self, size=1000, page=1, fields_to_return=[_ID]):
+        if size > 0:
+            result = self.post("allRecords?size=%s&page=%s" % (size, page), data={FIELDS_TO_RETURN: fields_to_return})
+        else:
+            result = self.post("allRecords", data={FIELDS_TO_RETURN: fields_to_return})
         return result
 
+    def get_all_ids(self):
+        """
+        Returns an array with records with "_id" fields.
+        :return: {"answer_records": [an array with records with "_id" fields], "answer_count": "# of records"}
+        """
+        return self.allRecords(size=0, fields_to_return=[_ID])
+
     def search(self, query_record, filter=None, k=5, fields_to_return=[_ID]):
         """
         Calls visual knn
@@ -78,6 +82,20 @@ class SimilarityPhotosClient(RestClient):
         data = self.construct_data(query_record, filter=filter, k=k, fields_to_return=fields_to_return)
         return self.post(self.PREDICT_ENDPOINT, data=data)
 
+    def search_and_rank(self, query_record, records, fields_to_return=[_ID]):
+        """
+        Ranks the records agains query
+        :param query_record: dictionary with field '_id' (from your collection) or '_url' or "_base64' data
+        :param k: how many similar items to return
+        :param fields_to_return: fields to return in every record
+        :param filter: how to filter picked items (mongodb syntax)
+        :return: json response
+        """
+        data = self.construct_data(query_record, fields_to_return=fields_to_return)
+        data[RECORDS] = self.preprocess_records(records)
+        del data[K_COUNT]
+        return self.post(RANK_RECORDS, data=data)
+
     def random(self, filter=None, count=10, fields_to_return=[_ID]):
         """
         Call random endpoint which returns random(X=count) records from your collection.
@@ -136,17 +154,38 @@ class SimilarityPhotosClient(RestClient):
 
 class SimilarityProductsClient(SimilarityPhotosClient):
     def __init__(
-        self,
-        token,
-        collection=None,
-        collection_id=None,
-        endpoint=ENDPOINT + SIMILARITY_PRODUCTS,
-        resource_name=PRODUCT_SIMILARITY,
+        self, token, collection_id=None, endpoint=ENDPOINT + SIMILARITY_PRODUCTS, resource_name=PRODUCT_SIMILARITY,
     ):
         super(SimilarityProductsClient, self).__init__(
-            token=token,
-            collection=collection,
-            collection_id=collection_id,
-            endpoint=endpoint,
-            resource_name=resource_name,
+            token=token, collection_id=collection_id, endpoint=endpoint, resource_name=resource_name,
         )
+
+
+class SimilarityProductsTagsClient(SimilarityPhotosClient):
+    def __init__(
+        self,
+        token,
+        collection_id=None,
+        endpoint=ENDPOINT + SIMILARITY_PRODUCTS_TAGS,
+        resource_name=PRODUCT_SIMILARITY_TAGS,
+    ):
+        super(SimilarityProductsTagsClient, self).__init__(
+            token=token, collection_id=collection_id, endpoint=endpoint, resource_name=resource_name,
+        )
+
+    def insert(self, records, custom_flow=None):
+        """
+        Insert records into collection with all meta information.
+        :param custom_flow: string ID of the flow that should be called during the insert (in extractor)
+        :param records: dictionary with your "_id" and with one of "_url", "_file" or "_base64" to extract descriptor.
+        :return: json response
+        """
+        records = self.preprocess_records(records)
+        data = {RECORDS: self.fill_data(records, custom_flow)}
+        return self.post(INSERT, data=data)
+
+    def fill_data(self, records, custom_flow):
+        if custom_flow is not None:
+            for r in records:
+                r["custom_flow"] = custom_flow
+        return records
